
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: IsAddOnLoaded, next, ShadowUF

local coroutine = coroutine
local math = math

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration ShadowedUnitFrames")

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function round(number)
	return math.floor(number * 10) / 10
end

local function Aura_SetVertexColor(self, r, g, b, a)
	if (not self.GetBorder) then return end
	if (round(r) == 0.6) then r, g, b, a = nil, nil, nil, nil end
	self:GetBorder():SetColor(r, g, b, a)
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	if not IsAddOnLoaded("ShadowedUnitFrames") then
		self:RegisterEvent("ADDON_LOADED")
		return
	end

	if not self:IsHooked(ShadowUF.modules.auras, "Update") then
		self:RawHook(ShadowUF.modules.auras, "Update", "Auras_Update")
	end

	if not self:IsHooked(ShadowUF.modules.fader, "Update") then
		self:RawHook(ShadowUF.modules.fader, "Update", "Fader_Update")
	end

	if not self:IsHooked(ShadowUF.modules.highlight, "Update") then
		self:RawHook(ShadowUF.modules.highlight, "Update", "Highlight_Update")
	end

	if not self:IsHooked(ShadowUF.modules.units, "CreateUnit") then
		self:RawHook(ShadowUF.modules.units, "CreateUnit", "Units_CreateUnit")
	end

	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:StyleFrames()
end

function M:OnDisable()
	self:UnstyleFrames()
end

-- Style all unit frames.
function M:StyleFrames()
	if ShadowUF then
		for frame in next, ShadowUF.Units.frameList do
			self:StyleFrame(frame)
		end
	end
end

-- Style a unit frame.
function M:StyleFrame(frame)
	B:CreateBorder(frame)
	self:StyleAuras(frame)
end

-- Remove styling from all unit frames.
function M:UnstyleFrames()
	if ShadowUF then
		for frame in next, ShadowUF.Units.frameList do
			self:UnstyleFrame(frame)
		end
	end
end

-- Remove styling from a unit frame.
function M:UnstyleFrame(frame)
	B:RemoveBorder(frame)
	self:UnstyleAuras(frame)
end

-- Iterate over all aura buttons in a unit frame.
function M:IterateAuras(frame)
	local auras = frame and frame.auras
	local buffs = auras and auras.buffs
	local debuffs = auras and auras.debuffs

	return coroutine.wrap(function()
		for i = 1, (buffs and buffs.totalAuras or 0) do
			coroutine.yield(buffs.buttons[i])
		end

		for i = 1, (debuffs and debuffs.totalAuras or 0) do
			coroutine.yield(debuffs.buttons[i])
		end
	end)
end

-- Style all aura buttons in a unit frame.
function M:StyleAuras(frame)
	for button in self:IterateAuras(frame) do
		self:StyleAura(button)
	end
end

-- Style an aura button.
function M:StyleAura(button)
	if (not button) then return end
	local border = button.border

	B:CreateBorder(button):SetPadding(1)

	border.GetBorder = button.GetBorder

	if not self:IsHooked(border, "SetVertexColor") then
		self:SecureHook(border, "SetVertexColor", Aura_SetVertexColor)
		Aura_SetVertexColor(border, border:GetVertexColor())
	end
end

-- Remove styling from all aura buttons in a unit frame.
function M:UnstyleAuras(frame)
	for aura in self:IterateAuras(frame) do
		self:UnstyleAura(aura)
	end
end

-- Remove style from an aura.
function M:UnstyleAura(aura)
	if (not aura) then return end
	self:Unhook(aura.border, "SetVertexColor")
	aura.border.GetBorder = nil
	B:RemoveBorder(aura)
end

-- Synchronize SixUI's border color with Shadowed Unit Frames.
function M:SetBorderColor(frame)
	if frame and frame.GetBorder and frame.highlight then
		if frame.highlight:IsShown() then
			frame:GetBorder():SetColor(frame.highlight.top:GetVertexColor())
		else
			frame:GetBorder():SetColor()
		end
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

-- Enable the module when the Shadowed Unit Frames addon is loaded.
function M:ADDON_LOADED(event, name)
	if name == "ShadowedUnitFrames" then
		self:UnregisterEvent("ADDON_LOADED")
		self:OnEnable()
	end
end

-- Reload Shadowed Unit Frame's tags when the Control key is pressed or released.
function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	if ShadowUF.tagFunc then
		ShadowUF.Tags:Reload()
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

-- Style newly created aura buttons.
function M:Auras_Update(object, frame, ...)
	self.hooks[ShadowUF.modules.auras].Update(object, frame, ...)
	self:StyleAuras(frame)
end

-- Decrease the fade time for unit frames.
function M:Fader_Update(object, frame, ...)
	self.hooks[ShadowUF.modules.fader].Update(object, frame, ...)
	frame.fader.fadeTime = 0
end

-- Synchronize SixUI's border color with Shadowed Unit Frames.
function M:Highlight_Update(object, frame, ...)
	self.hooks[ShadowUF.modules.highlight].Update(object, frame, ...)
	self:SetBorderColor(frame)
end

-- Style newly created unit frames.
function M:Units_CreateUnit(...)
	local frame = self.hooks[ShadowUF.modules.units].CreateUnit(...)
	self:StyleFrame(frame)
	return frame
end
